@@ -6,6 +6,7 @@
 
 #include "config.h"
 #include "adw-application.h"
+#include "adw-dialog.h"
 #include "adw-main-private.h"
 
 /**
@@ -21,6 +22,15 @@
  *
  * ## Automatic Resources
  *
+ * ### Shortcuts Dialog
+ *
+ * If there's a resource located at `shortcuts-dialog.ui` which defines an
+ * [class@ShortcutsDialog] with the ID `shortcuts_dialog`, `AdwApplication`
+ * will set up an `app.shortcuts` action that creates and presents this dialog,
+ * as well as a <kbd>Ctrl</kbd><kbd>?</kbd> accelerator for it.
+ *
+ * ### Stylesheet
+ *
  * `AdwApplication` will automatically load stylesheets located in the
  * application's resource base path (see
  * [method@Gio.Application.set_resource_base_path], if they're present.
@@ -37,6 +47,14 @@
  *
  * - `style-hc-dark.css` contains styles used when the system high contrast
  *   preference is enabled and [property@StyleManager:dark] is `TRUE`.
+ *
+ * :::note
+ *     `style.css` can contain styles for dark and high contrast appearance as
+ *     well, using media queries:
+ *
+ *     - `prefers-color-scheme: dark` for styles used only for dark appearance.
+ *     - `prefers-contrast: more` for styles used only when the system high
+ *       contrast preference is enabled.
  */
 
 typedef struct
@@ -45,6 +63,7 @@ typedef struct
   GtkStyleProvider *dark_style_provider;
   GtkStyleProvider *hc_style_provider;
   GtkStyleProvider *hc_dark_style_provider;
+  char *shortcuts_dialog_path;
 } AdwApplicationPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AdwApplication, adw_application, GTK_TYPE_APPLICATION)
@@ -57,6 +76,71 @@ enum {
 
 static GParamSpec *props[LAST_PROP];
 
+static GFile *
+get_resource_base_path_file (AdwApplication *self)
+{
+  const char *base_path;
+  char *base_uri;
+  GFile *base_file;
+
+  base_path = g_application_get_resource_base_path (G_APPLICATION (self));
+
+  if (base_path == NULL)
+    return NULL;
+
+  base_uri = g_strconcat ("resource://", base_path, NULL);
+  base_file = g_file_new_for_uri (base_uri);
+
+  g_free (base_uri);
+
+  return base_file;
+}
+
+static void
+disable_shortcuts_action (AdwApplication *self)
+{
+  GAction *action = g_action_map_lookup_action (G_ACTION_MAP (self), "shortcuts");
+
+  g_assert (G_IS_SIMPLE_ACTION (action));
+
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
+}
+
+static void
+shortcuts_action_cb (AdwApplication *self)
+{
+  AdwApplicationPrivate *priv = adw_application_get_instance_private (self);
+  GtkBuilder *builder;
+  GError *error = NULL;
+  GObject *dialog;
+  GtkWindow *window;
+
+  builder = gtk_builder_new ();
+
+  gtk_builder_add_from_resource (builder, priv->shortcuts_dialog_path, &error);
+
+  if (error) {
+    g_critical ("Failed to create shortcuts window: %s", error->message);
+    disable_shortcuts_action (self);
+    g_error_free (error);
+    g_object_unref (builder);
+    return;
+  }
+
+  dialog = gtk_builder_get_object (builder, "shortcuts_dialog");
+  if (!ADW_IS_DIALOG (dialog)) {
+    disable_shortcuts_action (self);
+    g_object_unref (builder);
+    return;
+  }
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (self));
+
+  adw_dialog_present (ADW_DIALOG (dialog), GTK_WIDGET (window));
+
+  g_object_unref (builder);
+}
+
 static inline void
 style_provider_set_enabled (GtkStyleProvider *provider,
                             gboolean          enabled)
@@ -88,6 +172,47 @@ update_stylesheet (AdwApplication *self)
 
   if (priv->hc_dark_style_provider)
     style_provider_set_enabled (priv->hc_dark_style_provider, is_hc && is_dark);
+
+  GtkInterfaceColorScheme color_scheme;
+  GtkInterfaceContrast contrast;
+
+  if (is_dark)
+    color_scheme = GTK_INTERFACE_COLOR_SCHEME_DARK;
+  else
+    color_scheme = GTK_INTERFACE_COLOR_SCHEME_LIGHT;
+
+  if (is_hc)
+    contrast = GTK_INTERFACE_CONTRAST_MORE;
+  else
+    contrast = GTK_INTERFACE_CONTRAST_NO_PREFERENCE;
+
+  if (priv->base_style_provider) {
+    g_object_set (priv->base_style_provider,
+                  "prefers-color-scheme", color_scheme,
+                  "prefers-contrast", contrast,
+                  NULL);
+  }
+
+  if (priv->dark_style_provider) {
+    g_object_set (priv->dark_style_provider,
+                  "prefers-color-scheme", color_scheme,
+                  "prefers-contrast", contrast,
+                  NULL);
+  }
+
+  if (priv->hc_style_provider) {
+    g_object_set (priv->hc_style_provider,
+                  "prefers-color-scheme", color_scheme,
+                  "prefers-contrast", contrast,
+                  NULL);
+  }
+
+  if (priv->hc_dark_style_provider) {
+    g_object_set (priv->hc_dark_style_provider,
+                  "prefers-color-scheme", color_scheme,
+                  "prefers-contrast", contrast,
+                  NULL);
+  }
 }
 
 static void
@@ -109,18 +234,11 @@ static void
 init_providers (AdwApplication *self)
 {
   AdwApplicationPrivate *priv = adw_application_get_instance_private (self);
-  const char *base_path;
-  char *base_uri;
-  GFile *base_file;
+  GFile *base_file = get_resource_base_path_file (self);
 
-  base_path = g_application_get_resource_base_path (G_APPLICATION (self));
-
-  if (base_path == NULL)
+  if (base_file == NULL)
     return;
 
-  base_uri = g_strconcat ("resource://", base_path, NULL);
-  base_file = g_file_new_for_uri (base_uri);
-
   if (!adw_is_granite_present ()) {
     init_provider_from_file (&priv->base_style_provider,
                              g_file_get_child (base_file, "style.css"));
@@ -133,7 +251,6 @@ init_providers (AdwApplication *self)
   }
 
   g_object_unref (base_file);
-  g_free (base_uri);
 }
 
 static void
@@ -168,6 +285,53 @@ init_styling (AdwApplication *self)
   update_stylesheet (self);
 }
 
+static void
+init_shortcuts_dialog (AdwApplication *self)
+{
+  AdwApplicationPrivate *priv = adw_application_get_instance_private (self);
+  GFile *base_file, *ui_file;
+  GSimpleAction *action;
+  char *uri;
+
+#ifdef __APPLE__
+  const char * const accels[] = { "<Meta>question", NULL };
+#else
+  const char * const accels[] = { "<Control>question", NULL };
+#endif
+
+  /* An app.shortcuts action already exists, so we don't add ours */
+  if (g_action_map_lookup_action (G_ACTION_MAP (self), "shortcuts") != NULL)
+    return;
+
+  base_file = get_resource_base_path_file (self);
+  if (base_file == NULL)
+    return;
+
+  ui_file = g_file_get_child (base_file, "shortcuts-dialog.ui");
+
+  if (!g_file_query_exists (ui_file, NULL)) {
+    g_object_unref (base_file);
+    g_object_unref (ui_file);
+    return;
+  }
+
+  uri = g_file_get_uri (ui_file);
+
+  /* Trim "resource://" to get a resource path */
+  priv->shortcuts_dialog_path = g_strdup (&uri[11]);
+
+  action = g_simple_action_new ("shortcuts", NULL);
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (shortcuts_action_cb), self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
+
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.shortcuts", accels);
+
+  g_free (uri);
+  g_object_unref (base_file);
+  g_object_unref (ui_file);
+  g_object_unref (action);
+}
+
 static void
 adw_application_startup (GApplication *application)
 {
@@ -179,6 +343,7 @@ adw_application_startup (GApplication *application)
 
   init_providers (self);
   init_styling (self);
+  init_shortcuts_dialog (self);
 }
 
 static void
@@ -195,6 +360,17 @@ adw_application_dispose (GObject *object)
   G_OBJECT_CLASS (adw_application_parent_class)->dispose (object);
 }
 
+static void
+adw_application_finalize (GObject *object)
+{
+  AdwApplication *self = ADW_APPLICATION (object);
+  AdwApplicationPrivate *priv = adw_application_get_instance_private (self);
+
+  g_free (priv->shortcuts_dialog_path);
+
+  G_OBJECT_CLASS (adw_application_parent_class)->finalize (object);
+}
+
 static void
 adw_application_get_property (GObject    *object,
                               guint       prop_id,
@@ -220,6 +396,7 @@ adw_application_class_init (AdwApplicationClass *klass)
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
   object_class->dispose = adw_application_dispose;
+  object_class->finalize = adw_application_finalize;
   object_class->get_property = adw_application_get_property;
 
   application_class->startup = adw_application_startup;
