@@ -155,6 +155,7 @@ struct _AdwBottomSheet
   int sheet_height;
   int bottom_bar_height;
 
+  gboolean handle_focus;
   GtkWidget *last_content_focus;
   GtkWidget *last_sheet_focus;
 
@@ -1669,7 +1670,7 @@ adw_bottom_sheet_set_open (AdwBottomSheet *self,
 
   root = gtk_widget_get_root (GTK_WIDGET (self));
 
-  if (gtk_widget_get_mapped (GTK_WIDGET (self))) {
+  if (gtk_widget_get_mapped (GTK_WIDGET (self)) && self->handle_focus) {
     if (root)
       focus = gtk_root_get_focus (root);
 
@@ -1689,7 +1690,7 @@ adw_bottom_sheet_set_open (AdwBottomSheet *self,
   if (self->modal)
     gtk_widget_set_can_focus (self->content_bin, !open);
 
-  if (gtk_widget_get_mapped (GTK_WIDGET (self))) {
+  if (gtk_widget_get_mapped (GTK_WIDGET (self)) && self->handle_focus) {
     if (open) {
       if (self->last_sheet_focus) {
         gtk_widget_grab_focus (self->last_sheet_focus);
@@ -2153,6 +2154,15 @@ adw_bottom_sheet_set_reveal_bottom_bar (AdwBottomSheet *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_REVEAL_BOTTOM_BAR]);
 }
 
+void
+adw_bottom_sheet_set_handle_focus (AdwBottomSheet *self,
+                                   gboolean        handle_focus)
+{
+  g_return_if_fail (ADW_IS_BOTTOM_SHEET (self));
+
+  self->handle_focus = !!handle_focus;
+}
+
 void
 adw_bottom_sheet_set_min_natural_width (AdwBottomSheet *self,
                                         int             min_natural_width)
