@@ -183,6 +183,12 @@ enum {
 
 static guint signals[SIGNAL_LAST_SIGNAL];
 
+static void
+child_breakpoint_bin_notify_current_breakpoint_cb (AdwDialog *self)
+{
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CURRENT_BREAKPOINT]);
+}
+
 static gboolean
 map_tick_cb (AdwDialog *self)
 {
@@ -504,6 +510,7 @@ update_presentation (AdwDialog *self)
     priv->bottom_sheet = ADW_BOTTOM_SHEET (adw_bottom_sheet_new ());
 
     adw_bottom_sheet_set_min_natural_width (priv->bottom_sheet, 360);
+    adw_bottom_sheet_set_handle_focus (priv->bottom_sheet, FALSE);
 
     if (!priv->first_map)
       adw_bottom_sheet_set_open (priv->bottom_sheet, TRUE);
@@ -658,13 +665,7 @@ present_as_window (AdwDialog *self,
 static gboolean
 activate_focus_cb (AdwDialog *self)
 {
-  AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
-  GtkRoot *root;
-
-  if (priv->window)
-    return GDK_EVENT_PROPAGATE;
-
-  root = gtk_widget_get_root (GTK_WIDGET (self));
+  GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
 
   if (!GTK_IS_WINDOW (root))
     return GDK_EVENT_PROPAGATE;
@@ -677,13 +678,7 @@ activate_focus_cb (AdwDialog *self)
 static gboolean
 activate_default_cb (AdwDialog *self)
 {
-  AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
-  GtkRoot *root;
-
-  if (priv->window)
-    return GDK_EVENT_PROPAGATE;
-
-  root = gtk_widget_get_root (GTK_WIDGET (self));
+  GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
 
   if (!GTK_IS_WINDOW (root))
     return GDK_EVENT_PROPAGATE;
@@ -1317,6 +1312,10 @@ adw_dialog_init (AdwDialog *self)
   g_object_bind_property (self, "height-request",
                           priv->child_breakpoint_bin, "height-request",
                           G_BINDING_DEFAULT);
+  g_signal_connect_swapped (priv->child_breakpoint_bin,
+                            "notify::current-breakpoint",
+                            G_CALLBACK (child_breakpoint_bin_notify_current_breakpoint_cb),
+                            self);
 }
 
 static void
@@ -1810,7 +1809,7 @@ adw_dialog_set_focus (AdwDialog *self,
   if (priv->focus_widget == focus)
     return;
 
-  if (!gtk_widget_get_can_focus (priv->bin)) {
+  if (!priv->bin || !gtk_widget_get_can_focus (priv->bin)) {
     g_set_weak_pointer (&priv->last_focus, priv->focus_widget);
     set_focus (self, focus);
     return;
