@@ -279,7 +279,8 @@ struct _AdwNavigationView
   gboolean transition_cancel;
   double transition_progress;
   gboolean gesture_active;
-  AdwNavigationDirection swipe_direction;
+  /* AdwNavigationDirection or -1 */
+  int swipe_direction;
 
   AdwShadowHelper *shadow_helper;
   AdwSwipeTracker *swipe_tracker;
@@ -341,6 +342,9 @@ adw_navigation_view_model_get_n_items (GListModel *model)
 {
   AdwNavigationViewModel *self = ADW_NAVIGATION_VIEW_MODEL (model);
 
+  if (G_UNLIKELY (!ADW_IS_NAVIGATION_VIEW (self->view)))
+    return 0;
+
   return g_list_model_get_n_items (G_LIST_MODEL (self->view->navigation_stack));
 }
 
@@ -350,6 +354,9 @@ adw_navigation_view_model_get_item (GListModel *model,
 {
   AdwNavigationViewModel *self = ADW_NAVIGATION_VIEW_MODEL (model);
 
+  if (G_UNLIKELY (!ADW_IS_NAVIGATION_VIEW (self->view)))
+    return NULL;
+
   return g_list_model_get_item (G_LIST_MODEL (self->view->navigation_stack),
                                 position);
 }
@@ -365,14 +372,27 @@ adw_navigation_view_model_list_model_init (GListModelInterface *iface)
 G_DEFINE_FINAL_TYPE_WITH_CODE (AdwNavigationViewModel, adw_navigation_view_model, G_TYPE_OBJECT,
                                G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, adw_navigation_view_model_list_model_init))
 
+static void
+adw_navigation_view_model_dispose (GObject *object)
+{
+  AdwNavigationViewModel *self = ADW_NAVIGATION_VIEW_MODEL (object);
+
+  g_clear_weak_pointer (&self->view);
+
+  G_OBJECT_CLASS (adw_navigation_view_model_parent_class)->dispose (object);
+}
+
 static void
 adw_navigation_view_model_init (AdwNavigationViewModel *self)
 {
 }
 
 static void
-adw_navigation_view_model_class_init (AdwNavigationViewModelClass *class)
+adw_navigation_view_model_class_init (AdwNavigationViewModelClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->dispose = adw_navigation_view_model_dispose;
 }
 
 static GListModel *
@@ -381,7 +401,7 @@ adw_navigation_view_model_new (AdwNavigationView *view)
   AdwNavigationViewModel *model;
 
   model = g_object_new (ADW_TYPE_NAVIGATION_VIEW_MODEL, NULL);
-  model->view = view;
+  g_set_weak_pointer (&model->view, view);
 
   return G_LIST_MODEL (model);
 }
@@ -892,6 +912,9 @@ pop_from_stack (AdwNavigationView *self,
 
   old_page = adw_navigation_view_get_visible_page (self);
 
+  if (page_to == old_page)
+    return;
+
   length = g_list_model_get_n_items (G_LIST_MODEL (self->navigation_stack));
 
   g_assert (g_list_store_find (self->navigation_stack, page_to, &pos));
