@@ -20,6 +20,10 @@
 #include "adw-bottom-sheet-private.h"
 #include "adw-floating-sheet-private.h"
 
+#ifdef GDK_WINDOWING_MACOS
+#include <gdk/macos/gdkmacos.h>
+#endif
+
 typedef struct _CompareInfo CompareInfo;
 
 enum Axis {
@@ -639,6 +643,11 @@ adw_decoration_layout_prefers_start (const char *layout)
   char **sides;
   int i;
 
+#ifdef GDK_WINDOWING_MACOS
+  if (GDK_IS_MACOS_DISPLAY (gdk_display_get_default ()))
+    return TRUE;
+#endif
+
   if (!layout || !strchr (layout, ':'))
     return FALSE;
 
@@ -785,3 +794,31 @@ adw_get_inspector_keybinding_enabled (void)
 
   return enabled;
 }
+
+gboolean
+adw_tab_on_vertical_keynav_failed (GtkWidget        *widget,
+                                   GtkDirectionType  direction)
+{
+  GtkWidget *toplevel = GTK_WIDGET (gtk_widget_get_root (widget));
+
+  if (!toplevel)
+    return FALSE;
+
+  if (direction != GTK_DIR_UP && direction != GTK_DIR_DOWN)
+    return FALSE;
+
+  return gtk_widget_child_focus (toplevel, direction == GTK_DIR_UP ?
+                                 GTK_DIR_TAB_BACKWARD : GTK_DIR_TAB_FORWARD);
+}
+
+char *
+adw_get_badge_text (guint badge_number)
+{
+  if (badge_number > 999)
+    return g_strdup ("999+");
+
+  if (!badge_number)
+    return g_strdup ("");
+
+  return g_strdup_printf ("%u", badge_number);
+}
